# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.common import tag
from openstack import resource


class L7Rule(resource.Resource, tag.TagMixin):
    resource_key = 'rule'
    resources_key = 'rules'
    base_path = '/lbaas/l7policies/%(l7policy_id)s/rules'

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True

    _query_mapping = resource.QueryParameters(
        'compare_type', 'created_at', 'invert', 'key', 'project_id',
        'provisioning_status', 'type', 'updated_at', 'rule_value',
        'operating_status', is_admin_state_up='admin_state_up',
        l7_policy_id='l7policy_id', **tag.TagMixin._tag_query_parameters
    )

    #: Properties
    #: The administrative state of the l7policy *Type: bool*
    is_admin_state_up = resource.Body('admin_state_up', type=bool)
    #: comparison type to be used with the value in this L7 rule.
    compare_type = resource.Body('compare_type')
    #: Timestamp when the L7 rule was created.
    created_at = resource.Body('created_at')
    #: inverts the logic of the rule if True
    #  (ie. perform a logical NOT on the rule)
    invert = resource.Body('invert', type=bool)
    #: The key to use for the comparison.
    key = resource.Body('key')
    #: The ID of the associated l7 policy
    l7_policy_id = resource.URI('l7policy_id')
    #: The operating status of this l7rule
    operating_status = resource.Body('operating_status')
    #: The ID of the project this l7policy is associated with.
    project_id = resource.Body('project_id')
    #: The provisioning status of this l7policy
    provisioning_status = resource.Body('provisioning_status')
    #: The type of L7 rule
    type = resource.Body('type')
    #: Timestamp when the L7 rule was updated.
    updated_at = resource.Body('updated_at')
    #: value to be compared with
    rule_value = resource.Body('value')



#   Copyright 2021 Huawei, Inc. All rights reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from openstack.network.v2 import local_ip
from openstack.tests.unit import base

IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'created_at': '0',
    'id': IDENTIFIER,
    'name': '1',
    'description': '2',
    'project_id': '3',
    'local_port_id': '4',
    'network_id': '5',
    'local_ip_address': '127.0.0.1',
    'ip_mode': 'translate',
    'revision_number': '6',
    'updated_at': '7',
}


class TestLocalIP(base.TestCase):

    def test_basic(self):
        sot = local_ip.LocalIP()
        self.assertEqual('local_ip', sot.resource_key)
        self.assertEqual('local_ips', sot.resources_key)
        self.assertEqual('/local_ips', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

        self.assertDictEqual({"name": "name",
                              "description": "description",
                              "project_id": "project_id",
                              "network_id": "network_id",
                              "local_port_id": "local_port_id",
                              "local_ip_address": "local_ip_address",
                              "ip_mode": "ip_mode",
                              "sort_key": "sort_key",
                              "sort_dir": "sort_dir",
                              "limit": "limit",
                              "marker": "marker"},
                             sot._query_mapping._mapping)

    def test_make_it(self):
        sot = local_ip.LocalIP(**EXAMPLE)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['local_port_id'], sot.local_port_id)
        self.assertEqual(EXAMPLE['network_id'], sot.network_id)
        self.assertEqual(EXAMPLE['local_ip_address'], sot.local_ip_address)
        self.assertEqual(EXAMPLE['ip_mode'], sot.ip_mode)
        self.assertEqual(EXAMPLE['revision_number'], sot.revision_number)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import exceptions
from openstack import resource
from openstack import utils


class GroupType(resource.Resource):
    resource_key = "group_type"
    resources_key = "group_types"
    base_path = "/group_types"

    # capabilities
    allow_fetch = True
    allow_create = True
    allow_delete = True
    allow_commit = True
    allow_list = True

    _max_microversion = "3.11"

    #: Properties
    #: The group type description.
    description = resource.Body("description")
    #: Contains the specifications for a group type.
    group_specs = resource.Body("group_specs", type=dict, default={})
    #: Whether the group type is publicly visible.
    is_public = resource.Body("is_public", type=bool)

    def fetch_group_specs(self, session):
        """Fetch group_specs of the group type.

        These are returned by default if the user has suitable permissions
        (i.e. you're an admin) but by default you also need the same
        permissions to access this API. That means this function is kind of
        useless. However, that is how the API was designed and it is
        theoretically possible that people will have modified their policy to
        allow this but not the other so we provide this anyway.

        :param session: The session to use for making this request.
        :returns: An updated version of this object.
        """
        url = utils.urljoin(GroupType.base_path, self.id, 'group_specs')
        microversion = self._get_microversion(session, action='fetch')
        response = session.get(url, microversion=microversion)
        exceptions.raise_from_response(response)
        specs = response.json().get('group_specs', {})
        self._update(group_specs=specs)
        return self

    def create_group_specs(self, session, specs):
        """Creates group specs for the group type.

        This will override whatever specs are already present on the group
        type.

        :param session: The session to use for making this request.
        :param specs: A dict of group specs to set on the group type.
        :returns: An updated version of this object.
        """
        url = utils.urljoin(GroupType.base_path, self.id, 'group_specs')
        microversion = self._get_microversion(session, action='create')
        response = session.post(
            url, json={'group_specs': specs}, microversion=microversion,
        )
        exceptions.raise_from_response(response)
        specs = response.json().get('group_specs', {})
        self._update(group_specs=specs)
        return self

    def get_group_specs_property(self, session, prop):
        """Retrieve a group spec property of the group type.

        :param session: The session to use for making this request.
        :param prop: The name of the group spec property to update.
        :returns: The value of the group spec property.
        """
        url = utils.urljoin(GroupType.base_path, self.id, 'group_specs', prop)
        microversion = self._get_microversion(session, action='fetch')
        response = session.get(url, microversion=microversion)
        exceptions.raise_from_response(response)
        val = response.json().get(prop)
        return val

    def update_group_specs_property(self, session, prop, val):
        """Update a group spec property of the group type.

        :param session: The session to use for making this request.
        :param prop: The name of the group spec property to update.
        :param val: The value to set for the group spec property.
        :returns: The updated value of the group spec property.
        """
        url = utils.urljoin(GroupType.base_path, self.id, 'group_specs', prop)
        microversion = self._get_microversion(session, action='commit')
        response = session.put(
            url, json={prop: val}, microversion=microversion
        )
        exceptions.raise_from_response(response)
        val = response.json().get(prop)
        return val

    def delete_group_specs_property(self, session, prop):
        """Delete a group spec property from the group type.

        :param session: The session to use for making this request.
        :param prop: The name of the group spec property to delete.
        :returns: None
        """
        url = utils.urljoin(GroupType.base_path, self.id, 'group_specs', prop)
        microversion = self._get_microversion(session, action='delete')
        response = session.delete(url, microversion=microversion)
        exceptions.raise_from_response(response)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.identity.v3 import credential
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'blob': '1',
    'id': IDENTIFIER,
    'project_id': '3',
    'type': '4',
    'user_id': '5',
}


class TestCredential(base.TestCase):

    def test_basic(self):
        sot = credential.Credential()
        self.assertEqual('credential', sot.resource_key)
        self.assertEqual('credentials', sot.resources_key)
        self.assertEqual('/credentials', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertEqual('PATCH', sot.commit_method)

        self.assertDictEqual(
            {
                'type': 'type',
                'user_id': 'user_id',
                'limit': 'limit',
                'marker': 'marker',
            },
            sot._query_mapping._mapping)

    def test_make_it(self):
        sot = credential.Credential(**EXAMPLE)
        self.assertEqual(EXAMPLE['blob'], sot.blob)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['type'], sot.type)
        self.assertEqual(EXAMPLE['user_id'], sot.user_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import cloud as openstack

# Initialize and turn on debug logging
openstack.enable_logging(debug=True)

for cloud_name, region_name, image, flavor_id in [
        ('my-vexxhost', 'ca-ymq-1', 'Ubuntu 16.04.1 LTS [2017-03-03]',
         '5cf64088-893b-46b5-9bb1-ee020277635d'),
        ('my-citycloud', 'Buf1', 'Ubuntu 16.04 Xenial Xerus',
         '0dab10b5-42a2-438e-be7b-505741a7ffcc'),
        ('my-internap', 'ams01', 'Ubuntu 16.04 LTS (Xenial Xerus)',
         'A1.4')]:
    # Initialize cloud
    cloud = openstack.connect(
        cloud=cloud_name, region_name=region_name)

    # Boot a server, wait for it to boot, and then do whatever is needed
    # to get a public ip for it.
    server = cloud.create_server(
        'my-server', image=image, flavor=dict(id=flavor_id),
        wait=True, auto_ip=True)
    # Delete it - this is a demo
    cloud.delete_server(server, wait=True, delete_ips=True)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class ServerDiagnostics(resource.Resource):
    resource_key = 'diagnostics'
    base_path = '/servers/%(server_id)s/diagnostics'

    # capabilities
    allow_fetch = True

    requires_id = False

    #: Indicates whether or not a config drive was used for this server.
    has_config_drive = resource.Body('config_drive')
    #: The current state of the VM.
    state = resource.Body('state')
    #: The driver on which the VM is running.
    driver = resource.Body('driver')
    #: The hypervisor on which the VM is running.
    hypervisor = resource.Body('hypervisor')
    #: The hypervisor OS.
    hypervisor_os = resource.Body('hypervisor_os')
    #: The amount of time in seconds that the VM has been running.
    uptime = resource.URI('uptime')
    #: The number of vCPUs.
    num_cpus = resource.URI('num_cpus')
    #: The number of disks.
    num_disks = resource.URI('num_disks')
    #: The number of vNICs.
    num_nics = resource.URI('num_nics')
    #: The dictionary with information about VM memory usage.
    memory_details = resource.URI('memory_details')
    #: The list of dictionaries with detailed information about VM CPUs.
    cpu_details = resource.URI('cpu_details')
    #: The list of dictionaries with detailed information about VM disks.
    disk_details = resource.URI('disk_details')
    #: The list of dictionaries with detailed information about VM NICs.
    nic_details = resource.URI('nic_details')
    #: The ID for the server.
    server_id = resource.URI('server_id')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
List resources from the Compute service.

For a full guide see
https://docs.openstack.org/openstacksdk/latest/user/guides/compute.html
"""


def list_servers(conn):
    print("List Servers:")

    for server in conn.compute.servers():
        print(server)


def list_images(conn):
    print("List Images:")

    for image in conn.compute.images():
        print(image)


def list_flavors(conn):
    print("List Flavors:")

    for flavor in conn.compute.flavors():
        print(flavor)


def list_keypairs(conn):
    print("List Keypairs:")

    for keypair in conn.compute.keypairs():
        print(keypair)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class StackTemplate(resource.Resource):

    base_path = "/stacks/%(stack_name)s/%(stack_id)s/template"

    # capabilities
    allow_create = False
    allow_list = False
    allow_fetch = True
    allow_delete = False
    allow_commit = False

    # Properties
    #: Name of the stack where the template is referenced.
    name = resource.URI('stack_name')
    # Backwards compat. _stack_name will never match, but the alias will
    # point it to the value pulled for name.
    stack_name = resource.URI('_stack_name', alias='name')
    #: ID of the stack where the template is referenced.
    stack_id = resource.URI('stack_id', alternate_id=True)
    #: The description specified in the template
    description = resource.Body('Description')
    #: The version of the orchestration HOT template.
    heat_template_version = resource.Body('heat_template_version')
    #: Key and value that contain output data.
    outputs = resource.Body('outputs', type=dict)
    #: Key and value pairs that contain template parameters
    parameters = resource.Body('parameters', type=dict)
    #: Key and value pairs that contain definition of resources in the
    #: template
    resources = resource.Body('resources', type=dict)
    # List parameters grouped.
    parameter_groups = resource.Body('parameter_groups', type=list)
    # Restrict conditions which supported since '2016-10-14'.
    conditions = resource.Body('conditions', type=dict)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from openstack.load_balancer.v2 import availability_zone_profile
from openstack.tests.unit import base


IDENTIFIER = uuid.uuid4()
EXAMPLE = {
    'id': IDENTIFIER,
    'name': 'acidic',
    'provider_name': 'best',
    'availability_zone_data': '{"loadbalancer_topology": "SINGLE"}'}


class TestAvailabilityZoneProfile(base.TestCase):

    def test_basic(self):
        test_profile = availability_zone_profile.AvailabilityZoneProfile()
        self.assertEqual('availability_zone_profile',
                         test_profile.resource_key)
        self.assertEqual('availability_zone_profiles',
                         test_profile.resources_key)
        self.assertEqual('/lbaas/availabilityzoneprofiles',
                         test_profile.base_path)
        self.assertTrue(test_profile.allow_create)
        self.assertTrue(test_profile.allow_fetch)
        self.assertTrue(test_profile.allow_commit)
        self.assertTrue(test_profile.allow_delete)
        self.assertTrue(test_profile.allow_list)

    def test_make_it(self):
        test_profile = availability_zone_profile.AvailabilityZoneProfile(
            **EXAMPLE)
        self.assertEqual(EXAMPLE['id'], test_profile.id)
        self.assertEqual(EXAMPLE['name'], test_profile.name)
        self.assertEqual(EXAMPLE['provider_name'], test_profile.provider_name)
        self.assertEqual(EXAMPLE['availability_zone_data'],
                         test_profile.availability_zone_data)

        self.assertDictEqual(
            {'limit': 'limit',
             'marker': 'marker',
             'id': 'id',
             'name': 'name',
             'provider_name': 'provider_name',
             'availability_zone_data': 'availability_zone_data'},
            test_profile._query_mapping._mapping)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import subnet
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'allocation_pools': [{'1': 1}],
    'cidr': '2',
    'created_at': '3',
    'description': '4',
    'dns_nameservers': ['5'],
    'dns_publish_fixed_ip': True,
    'enable_dhcp': True,
    'gateway_ip': '6',
    'host_routes': ['7'],
    'id': IDENTIFIER,
    'ip_version': 8,
    'ipv6_address_mode': '9',
    'ipv6_ra_mode': '10',
    'name': '11',
    'network_id': '12',
    'revision_number': 13,
    'segment_id': '14',
    'service_types': ['15'],
    'subnetpool_id': '16',
    'project_id': '17',
    'updated_at': '18',
    'use_default_subnetpool': True,
}


class TestSubnet(base.TestCase):

    def test_basic(self):
        sot = subnet.Subnet()
        self.assertEqual('subnet', sot.resource_key)
        self.assertEqual('subnets', sot.resources_key)
        self.assertEqual('/subnets', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = subnet.Subnet(**EXAMPLE)
        self.assertEqual(EXAMPLE['allocation_pools'], sot.allocation_pools)
        self.assertEqual(EXAMPLE['cidr'], sot.cidr)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['dns_nameservers'], sot.dns_nameservers)
        self.assertTrue(sot.dns_publish_fixed_ip)
        self.assertTrue(sot.is_dhcp_enabled)
        self.assertEqual(EXAMPLE['gateway_ip'], sot.gateway_ip)
        self.assertEqual(EXAMPLE['host_routes'], sot.host_routes)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['ip_version'], sot.ip_version)
        self.assertEqual(EXAMPLE['ipv6_address_mode'], sot.ipv6_address_mode)
        self.assertEqual(EXAMPLE['ipv6_ra_mode'], sot.ipv6_ra_mode)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['network_id'], sot.network_id)
        self.assertEqual(EXAMPLE['revision_number'], sot.revision_number)
        self.assertEqual(EXAMPLE['segment_id'], sot.segment_id)
        self.assertEqual(EXAMPLE['service_types'], sot.service_types)
        self.assertEqual(EXAMPLE['subnetpool_id'], sot.subnet_pool_id)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)
        self.assertTrue(sot.use_default_subnet_pool)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class QoSDSCPMarkingRule(resource.Resource):
    resource_key = 'dscp_marking_rule'
    resources_key = 'dscp_marking_rules'
    base_path = '/qos/policies/%(qos_policy_id)s/dscp_marking_rules'

    _allow_unknown_attrs_in_body = True

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    # Properties
    #: The ID of the QoS policy who owns rule.
    qos_policy_id = resource.URI('qos_policy_id')
    #: DSCP mark field.
    dscp_mark = resource.Body('dscp_mark')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock
import uuid

import testtools

from openstack.cloud import exc
from openstack.config import cloud_region
from openstack.tests import fakes
from openstack.tests.unit import base


class TestOperatorCloud(base.TestCase):

    def test_get_image_name(self):
        self.use_glance()

        image_id = self.getUniqueString()
        fake_image = fakes.make_fake_image(image_id=image_id)
        list_return = {'images': [fake_image]}

        self.register_uris([
            dict(method='GET',
                 uri='https://image.example.com/v2/images',
                 json=list_return),
            dict(method='GET',
                 uri='https://image.example.com/v2/images',
                 json=list_return),
        ])

        self.assertEqual('fake_image', self.cloud.get_image_name(image_id))
        self.assertEqual('fake_image', self.cloud.get_image_name('fake_image'))

        self.assert_calls()

    def test_get_image_id(self):
        self.use_glance()

        image_id = self.getUniqueString()
        fake_image = fakes.make_fake_image(image_id=image_id)
        list_return = {'images': [fake_image]}

        self.register_uris([
            dict(method='GET',
                 uri='https://image.example.com/v2/images',
                 json=list_return),
            dict(method='GET',
                 uri='https://image.example.com/v2/images',
                 json=list_return),
        ])

        self.assertEqual(image_id, self.cloud.get_image_id(image_id))
        self.assertEqual(image_id, self.cloud.get_image_id('fake_image'))

        self.assert_calls()

    @mock.patch.object(cloud_region.CloudRegion, 'get_session')
    def test_get_session_endpoint_exception(self, get_session_mock):
        class FakeException(Exception):
            pass

        def side_effect(*args, **kwargs):
            raise FakeException("No service")
        session_mock = mock.Mock()
        session_mock.get_endpoint.side_effect = side_effect
        get_session_mock.return_value = session_mock
        self.cloud.name = 'testcloud'
        self.cloud.config.config['region_name'] = 'testregion'
        with testtools.ExpectedException(
                exc.OpenStackCloudException,
                "Error getting image endpoint on testcloud:testregion:"
                " No service"):
            self.cloud.get_session_endpoint("image")

    @mock.patch.object(cloud_region.CloudRegion, 'get_session')
    def test_get_session_endpoint_unavailable(self, get_session_mock):
        session_mock = mock.Mock()
        session_mock.get_endpoint.return_value = None
        get_session_mock.return_value = session_mock
        image_endpoint = self.cloud.get_session_endpoint("image")
        self.assertIsNone(image_endpoint)

    @mock.patch.object(cloud_region.CloudRegion, 'get_session')
    def test_get_session_endpoint_identity(self, get_session_mock):
        session_mock = mock.Mock()
        get_session_mock.return_value = session_mock
        self.cloud.get_session_endpoint('identity')
        kwargs = dict(
            interface='public', region_name='RegionOne',
            service_name=None, service_type='identity')

        session_mock.get_endpoint.assert_called_with(**kwargs)

    @mock.patch.object(cloud_region.CloudRegion, 'get_session')
    def test_has_service_no(self, get_session_mock):
        session_mock = mock.Mock()
        session_mock.get_endpoint.return_value = None
        get_session_mock.return_value = session_mock
        self.assertFalse(self.cloud.has_service("image"))

    @mock.patch.object(cloud_region.CloudRegion, 'get_session')
    def test_has_service_yes(self, get_session_mock):
        session_mock = mock.Mock()
        session_mock.get_endpoint.return_value = 'http://fake.url'
        get_session_mock.return_value = session_mock
        self.assertTrue(self.cloud.has_service("image"))

    def test_list_hypervisors(self):
        '''This test verifies that calling list_hypervisors results in a call
        to nova client.'''
        uuid1 = uuid.uuid4().hex
        uuid2 = uuid.uuid4().hex
        self.use_compute_discovery()
        self.register_uris([
            dict(
                method='GET',
                uri='https://compute.example.com/v2.1/os-hypervisors/detail',
                json={
                    'hypervisors': [
                        fakes.make_fake_hypervisor(uuid1, 'testserver1'),
                        fakes.make_fake_hypervisor(uuid2, 'testserver2'),
                    ]
                },
                validate={
                    'headers': {
                        'OpenStack-API-Version': 'compute 2.53'
                    }
                }
            ),
        ])

        r = self.cloud.list_hypervisors()

        self.assertEqual(2, len(r))
        self.assertEqual('testserver1', r[0]['name'])
        self.assertEqual(uuid1, r[0]['id'])
        self.assertEqual('testserver2', r[1]['name'])
        self.assertEqual(uuid2, r[1]['id'])

        self.assert_calls()

    def test_list_old_hypervisors(self):
        '''This test verifies that calling list_hypervisors on a pre-2.53 cloud
        calls the old version.'''
        self.use_compute_discovery(
            compute_version_json='old-compute-version.json')
        self.register_uris([
            dict(
                method='GET',
                uri='https://compute.example.com/v2.1/os-hypervisors/detail',
                json={
                    'hypervisors': [
                        fakes.make_fake_hypervisor('1', 'testserver1'),
                        fakes.make_fake_hypervisor('2', 'testserver2'),
                    ]
                }
            ),
        ])

        r = self.cloud.list_hypervisors()

        self.assertEqual(2, len(r))
        self.assertEqual('testserver1', r[0]['name'])
        self.assertEqual('testserver2', r[1]['name'])

        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_compute
----------------------------------

Functional tests for `shade` compute methods.
"""

import datetime

from fixtures import TimeoutException

from openstack.cloud import exc
from openstack.tests.functional import base
from openstack import utils


class TestCompute(base.BaseFunctionalTest):
    def setUp(self):
        # OS_TEST_TIMEOUT is 90 sec by default
        # but on a bad day, test_attach_detach_volume can take more time.
        self.TIMEOUT_SCALING_FACTOR = 1.5

        super(TestCompute, self).setUp()
        self.server_name = self.getUniqueString()

    def _cleanup_servers_and_volumes(self, server_name):
        """Delete the named server and any attached volumes.

        Adding separate cleanup calls for servers and volumes can be tricky
        since they need to be done in the proper order. And sometimes deleting
        a server can start the process of deleting a volume if it is booted
        from that volume. This encapsulates that logic.
        """
        server = self.user_cloud.get_server(server_name)
        if not server:
            return
        volumes = self.user_cloud.get_volumes(server)
        try:
            self.user_cloud.delete_server(server.name, wait=True)
            for volume in volumes:
                if volume.status != 'deleting':
                    self.user_cloud.delete_volume(volume.id, wait=True)
        except (exc.OpenStackCloudTimeout, TimeoutException):
            # Ups, some timeout occured during process of deletion server
            # or volumes, so now we will try to call delete each of them
            # once again and we will try to live with it
            self.user_cloud.delete_server(server.name)
            for volume in volumes:
                self.operator_cloud.delete_volume(
                    volume.id, wait=False, force=True)

    def test_create_and_delete_server(self):
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        server = self.user_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            wait=True)
        self.assertEqual(self.server_name, server['name'])
        self.assertEqual(self.image.id, server['image']['id'])
        self.assertEqual(self.flavor.name, server['flavor']['original_name'])
        self.assertIsNotNone(server['adminPass'])
        self.assertTrue(
            self.user_cloud.delete_server(self.server_name, wait=True))
        srv = self.user_cloud.get_server(self.server_name)
        self.assertTrue(srv is None or srv.status.lower() == 'deleted')

    def test_create_and_delete_server_auto_ip_delete_ips(self):
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        server = self.user_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            auto_ip=True,
            wait=True)
        self.assertEqual(self.server_name, server['name'])
        self.assertEqual(self.image.id, server['image']['id'])
        self.assertEqual(self.flavor.name, server['flavor']['original_name'])
        self.assertIsNotNone(server['adminPass'])
        self.assertTrue(
            self.user_cloud.delete_server(
                self.server_name, wait=True, delete_ips=True))
        srv = self.user_cloud.get_server(self.server_name)
        self.assertTrue(srv is None or srv.status.lower() == 'deleted')

    def test_attach_detach_volume(self):
        self.skipTest('Volume functional tests temporarily disabled')
        server_name = self.getUniqueString()
        self.addCleanup(self._cleanup_servers_and_volumes, server_name)
        server = self.user_cloud.create_server(
            name=server_name, image=self.image, flavor=self.flavor,
            wait=True)
        volume = self.user_cloud.create_volume(1)
        vol_attachment = self.user_cloud.attach_volume(server, volume)
        for key in ('device', 'serverId', 'volumeId'):
            self.assertIn(key, vol_attachment)
            self.assertTrue(vol_attachment[key])  # assert string is not empty
        self.assertIsNone(self.user_cloud.detach_volume(server, volume))

    def test_create_and_delete_server_with_config_drive(self):
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        server = self.user_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            config_drive=True,
            wait=True)
        self.assertEqual(self.server_name, server['name'])
        self.assertEqual(self.image.id, server['image']['id'])
        self.assertEqual(self.flavor.name, server['flavor']['original_name'])
        self.assertTrue(server['has_config_drive'])
        self.assertIsNotNone(server['adminPass'])
        self.assertTrue(
            self.user_cloud.delete_server(self.server_name, wait=True))
        srv = self.user_cloud.get_server(self.server_name)
        self.assertTrue(srv is None or srv.status.lower() == 'deleted')

    def test_create_and_delete_server_with_config_drive_none(self):
        # check that we're not sending invalid values for config_drive
        # if it's passed in explicitly as None - which nodepool does if it's
        # not set in the config
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        server = self.user_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            config_drive=None,
            wait=True)
        self.assertEqual(self.server_name, server['name'])
        self.assertEqual(self.image.id, server['image']['id'])
        self.assertEqual(self.flavor.name, server['flavor']['original_name'])
        self.assertFalse(server['has_config_drive'])
        self.assertIsNotNone(server['adminPass'])
        self.assertTrue(
            self.user_cloud.delete_server(
                self.server_name, wait=True))
        srv = self.user_cloud.get_server(self.server_name)
        self.assertTrue(srv is None or srv.status.lower() == 'deleted')

    def test_list_all_servers(self):
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        server = self.user_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            wait=True)
        # We're going to get servers from other tests, but that's ok, as long
        # as we get the server we created with the demo user.
        found_server = False
        for s in self.operator_cloud.list_servers(all_projects=True):
            if s.name == server.name:
                found_server = True
        self.assertTrue(found_server)

    def test_list_all_servers_bad_permissions(self):
        # Normal users are not allowed to pass all_projects=True
        self.assertRaises(
            exc.OpenStackCloudException,
            self.user_cloud.list_servers,
            all_projects=True)

    def test_create_server_image_flavor_dict(self):
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        server = self.user_cloud.create_server(
            name=self.server_name,
            image={'id': self.image.id},
            flavor={'id': self.flavor.id},
            wait=True)
        self.assertEqual(self.server_name, server['name'])
        self.assertEqual(self.image.id, server['image']['id'])
        self.assertEqual(self.flavor.name, server['flavor']['original_name'])
        self.assertIsNotNone(server['adminPass'])
        self.assertTrue(
            self.user_cloud.delete_server(self.server_name, wait=True))
        srv = self.user_cloud.get_server(self.server_name)
        self.assertTrue(srv is None or srv.status.lower() == 'deleted')

    def test_get_server_console(self):
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        server = self.user_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            wait=True)
        # _get_server_console_output does not trap HTTP exceptions, so this
        # returning a string tests that the call is correct. Testing that
        # the cloud returns actual data in the output is out of scope.
        log = self.user_cloud._get_server_console_output(server_id=server.id)
        self.assertTrue(isinstance(log, str))

    def test_get_server_console_name_or_id(self):
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        self.user_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            wait=True)
        log = self.user_cloud.get_server_console(server=self.server_name)
        self.assertTrue(isinstance(log, str))

    def test_list_availability_zone_names(self):
        self.assertEqual(
            ['nova'], self.user_cloud.list_availability_zone_names())

    def test_get_server_console_bad_server(self):
        self.assertRaises(
            exc.OpenStackCloudException,
            self.user_cloud.get_server_console,
            server=self.server_name)

    def test_create_and_delete_server_with_admin_pass(self):
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        server = self.user_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            admin_pass='sheiqu9loegahSh',
            wait=True)
        self.assertEqual(self.server_name, server['name'])
        self.assertEqual(self.image.id, server['image']['id'])
        self.assertEqual(self.flavor.name, server['flavor']['original_name'])
        self.assertEqual(server['adminPass'], 'sheiqu9loegahSh')
        self.assertTrue(
            self.user_cloud.delete_server(self.server_name, wait=True))
        srv = self.user_cloud.get_server(self.server_name)
        self.assertTrue(srv is None or srv.status.lower() == 'deleted')

    def test_get_image_id(self):
        self.assertEqual(
            self.image.id, self.user_cloud.get_image_id(self.image.id))
        self.assertEqual(
            self.image.id, self.user_cloud.get_image_id(self.image.name))

    def test_get_image_name(self):
        self.assertEqual(
            self.image.name, self.user_cloud.get_image_name(self.image.id))
        self.assertEqual(
            self.image.name, self.user_cloud.get_image_name(self.image.name))

    def _assert_volume_attach(self, server, volume_id=None, image=''):
        self.assertEqual(self.server_name, server['name'])
        self.assertEqual(image, server['image'])
        self.assertEqual(self.flavor.id, server['flavor']['id'])
        volumes = self.user_cloud.get_volumes(server)
        self.assertEqual(1, len(volumes))
        volume = volumes[0]
        if volume_id:
            self.assertEqual(volume_id, volume['id'])
        else:
            volume_id = volume['id']
        self.assertEqual(1, len(volume['attachments']), 1)
        self.assertEqual(server['id'], volume['attachments'][0]['server_id'])
        return volume_id

    def test_create_boot_from_volume_image(self):
        self.skipTest('Volume functional tests temporarily disabled')
        if not self.user_cloud.has_service('volume'):
            self.skipTest('volume service not supported by cloud')
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        server = self.user_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            boot_from_volume=True,
            volume_size=1,
            wait=True)
        volume_id = self._assert_volume_attach(server)
        volume = self.user_cloud.get_volume(volume_id)
        self.assertIsNotNone(volume)
        self.assertEqual(volume['name'], volume['display_name'])
        self.assertTrue(volume['bootable'])
        self.assertEqual(server['id'], volume['attachments'][0]['server_id'])
        self.assertTrue(self.user_cloud.delete_server(server.id, wait=True))
        self._wait_for_detach(volume.id)
        self.assertTrue(self.user_cloud.delete_volume(volume.id, wait=True))
        srv = self.user_cloud.get_server(self.server_name)
        self.assertTrue(srv is None or srv.status.lower() == 'deleted')
        self.assertIsNone(self.user_cloud.get_volume(volume.id))

    def _wait_for_detach(self, volume_id):
        # Volumes do not show up as unattached for a bit immediately after
        # deleting a server that had had a volume attached. Yay for eventual
        # consistency!
        for count in utils.iterate_timeout(
                60,
                'Timeout waiting for volume {volume_id} to detach'.format(
                    volume_id=volume_id)):
            volume = self.user_cloud.get_volume(volume_id)
            if volume.status in (
                    'available', 'error',
                    'error_restoring', 'error_extending'):
                return

    def test_create_terminate_volume_image(self):
        self.skipTest('Volume functional tests temporarily disabled')
        if not self.user_cloud.has_service('volume'):
            self.skipTest('volume service not supported by cloud')
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        server = self.user_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            boot_from_volume=True,
            terminate_volume=True,
            volume_size=1,
            wait=True)
        volume_id = self._assert_volume_attach(server)
        self.assertTrue(
            self.user_cloud.delete_server(self.server_name, wait=True))
        volume = self.user_cloud.get_volume(volume_id)
        # We can either get None (if the volume delete was quick), or a volume
        # that is in the process of being deleted.
        if volume:
            self.assertEqual('deleting', volume.status)
        srv = self.user_cloud.get_server(self.server_name)
        self.assertTrue(srv is None or srv.status.lower() == 'deleted')

    def test_create_boot_from_volume_preexisting(self):
        self.skipTest('Volume functional tests temporarily disabled')
        if not self.user_cloud.has_service('volume'):
            self.skipTest('volume service not supported by cloud')
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        volume = self.user_cloud.create_volume(
            size=1, name=self.server_name, image=self.image, wait=True)
        self.addCleanup(self.user_cloud.delete_volume, volume.id)
        server = self.user_cloud.create_server(
            name=self.server_name,
            image=None,
            flavor=self.flavor,
            boot_volume=volume,
            volume_size=1,
            wait=True)
        volume_id = self._assert_volume_attach(server, volume_id=volume['id'])
        self.assertTrue(
            self.user_cloud.delete_server(self.server_name, wait=True))
        volume = self.user_cloud.get_volume(volume_id)
        self.assertIsNotNone(volume)
        self.assertEqual(volume['name'], volume['display_name'])
        self.assertTrue(volume['bootable'])
        self.assertEqual([], volume['attachments'])
        self._wait_for_detach(volume.id)
        self.assertTrue(self.user_cloud.delete_volume(volume_id))
        srv = self.user_cloud.get_server(self.server_name)
        self.assertTrue(srv is None or srv.status.lower() == 'deleted')
        self.assertIsNone(self.user_cloud.get_volume(volume_id))

    def test_create_boot_attach_volume(self):
        self.skipTest('Volume functional tests temporarily disabled')
        if not self.user_cloud.has_service('volume'):
            self.skipTest('volume service not supported by cloud')
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        volume = self.user_cloud.create_volume(
            size=1, name=self.server_name, image=self.image, wait=True)
        self.addCleanup(self.user_cloud.delete_volume, volume['id'])
        server = self.user_cloud.create_server(
            name=self.server_name,
            flavor=self.flavor,
            image=self.image,
            boot_from_volume=False,
            volumes=[volume],
            wait=True)
        volume_id = self._assert_volume_attach(
            server, volume_id=volume['id'], image={'id': self.image['id']})
        self.assertTrue(
            self.user_cloud.delete_server(self.server_name, wait=True))
        volume = self.user_cloud.get_volume(volume_id)
        self.assertIsNotNone(volume)
        self.assertEqual(volume['name'], volume['display_name'])
        self.assertEqual([], volume['attachments'])
        self._wait_for_detach(volume.id)
        self.assertTrue(self.user_cloud.delete_volume(volume_id))
        srv = self.user_cloud.get_server(self.server_name)
        self.assertTrue(srv is None or srv.status.lower() == 'deleted')
        self.assertIsNone(self.user_cloud.get_volume(volume_id))

    def test_create_boot_from_volume_preexisting_terminate(self):
        self.skipTest('Volume functional tests temporarily disabled')
        if not self.user_cloud.has_service('volume'):
            self.skipTest('volume service not supported by cloud')
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        volume = self.user_cloud.create_volume(
            size=1, name=self.server_name, image=self.image, wait=True)
        server = self.user_cloud.create_server(
            name=self.server_name,
            image=None,
            flavor=self.flavor,
            boot_volume=volume,
            terminate_volume=True,
            volume_size=1,
            wait=True)
        volume_id = self._assert_volume_attach(server, volume_id=volume['id'])
        self.assertTrue(
            self.user_cloud.delete_server(self.server_name, wait=True))
        volume = self.user_cloud.get_volume(volume_id)
        # We can either get None (if the volume delete was quick), or a volume
        # that is in the process of being deleted.
        if volume:
            self.assertEqual('deleting', volume.status)
        srv = self.user_cloud.get_server(self.server_name)
        self.assertTrue(srv is None or srv.status.lower() == 'deleted')

    def test_create_image_snapshot_wait_active(self):
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        server = self.user_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            admin_pass='sheiqu9loegahSh',
            wait=True)
        image = self.user_cloud.create_image_snapshot('test-snapshot', server,
                                                      wait=True)
        self.addCleanup(self.user_cloud.delete_image, image['id'])
        self.assertEqual('active', image['status'])

    def test_set_and_delete_metadata(self):
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        self.user_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            wait=True)
        self.user_cloud.set_server_metadata(self.server_name,
                                            {'key1': 'value1',
                                             'key2': 'value2'})
        updated_server = self.user_cloud.get_server(self.server_name)
        self.assertEqual(set(updated_server.metadata.items()),
                         set({'key1': 'value1', 'key2': 'value2'}.items()))

        self.user_cloud.set_server_metadata(self.server_name,
                                            {'key2': 'value3'})
        updated_server = self.user_cloud.get_server(self.server_name)
        self.assertEqual(set(updated_server.metadata.items()),
                         set({'key1': 'value1', 'key2': 'value3'}.items()))

        self.user_cloud.delete_server_metadata(self.server_name, ['key2'])
        updated_server = self.user_cloud.get_server(self.server_name)
        self.assertEqual(set(updated_server.metadata.items()),
                         set({'key1': 'value1'}.items()))

        self.user_cloud.delete_server_metadata(self.server_name, ['key1'])
        updated_server = self.user_cloud.get_server(self.server_name)
        self.assertEqual(set(updated_server.metadata.items()), set([]))

        self.assertRaises(
            exc.OpenStackCloudURINotFound,
            self.user_cloud.delete_server_metadata,
            self.server_name, ['key1'])

    def test_update_server(self):
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        self.user_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            wait=True)
        server_updated = self.user_cloud.update_server(
            self.server_name,
            name='new_name'
        )
        self.assertEqual('new_name', server_updated['name'])

    def test_get_compute_usage(self):
        '''Test usage functionality'''
        # Add a server so that we can know we have usage
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        self.user_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            wait=True)
        start = datetime.datetime.now() - datetime.timedelta(seconds=5)
        usage = self.operator_cloud.get_compute_usage('demo', start)
        self.add_info_on_exception('usage', usage)
        self.assertIsNotNone(usage)
        self.assertIn('total_hours', usage)
        self.assertIn('started_at', usage)
        self.assertEqual(start.isoformat(), usage['started_at'])
        self.assertIn('location', usage)



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import datetime
import uuid

from openstack.tests.unit import base


class TestUsage(base.TestCase):

    def test_get_usage(self):
        project = self.mock_for_keystone_projects(project_count=1,
                                                  list_get=True)[0]
        start = end = datetime.datetime.now()

        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public',
                     append=['os-simple-tenant-usage', project.project_id],
                     qs_elements=[
                         'start={now}'.format(now=start.isoformat()),
                         'end={now}'.format(now=end.isoformat()),
                     ]),
                 json={"tenant_usage": {
                     "server_usages": [
                         {
                             "ended_at": None,
                             "flavor": "m1.tiny",
                             "hours": 1.0,
                             "instance_id": uuid.uuid4().hex,
                             "local_gb": 1,
                             "memory_mb": 512,
                             "name": "instance-2",
                             "started_at": "2012-10-08T20:10:44.541277",
                             "state": "active",
                             "tenant_id": "6f70656e737461636b20342065766572",
                             "uptime": 3600,
                             "vcpus": 1
                         }
                     ],
                     "start": "2012-10-08T20:10:44.587336",
                     "stop": "2012-10-08T21:10:44.587336",
                     "tenant_id": "6f70656e737461636b20342065766572",
                     "total_hours": 1.0,
                     "total_local_gb_usage": 1.0,
                     "total_memory_mb_usage": 512.0,
                     "total_vcpus_usage": 1.0
                 }})
        ])

        self.cloud.get_compute_usage(project.project_id, start, end)

        self.assert_calls()



# Copyright(c) 2018 Nippon Telegraph and Telephone Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from openstack.instance_ha.v1 import notification
from openstack.tests.unit import base

FAKE_ID = "569429e9-7f14-41be-a38e-920277e637db"
FAKE_UUID = "a0e70d3a-b3a2-4616-b65d-a7c03a2c85fc"
FAKE_HOST_UUID = "cad9ff01-c354-4414-ba3c-31b925be67f1"
PAYLOAD = {
    "instance_uuid": "4032bc1d-d723-47f6-b5ac-b9b3e6dbb795",
    "vir_domain_event": "STOPPED_FAILED",
    "event": "LIFECYCLE"
}

PROGRESS_DETAILS = [{"timestamp": "2019-02-28 07:21:33.291810",
                     "progress": 1.0,
                     "message": "Skipping recovery for process "
                                "nova-compute as it is already disabled"}]

RECOVERY_WORKFLOW_DETAILS = [{"progress": 1.0, "state": "SUCCESS",
                              "name": "DisableComputeNodeTask",
                              "progress_details": PROGRESS_DETAILS}]

NOTIFICATION = {
    "id": FAKE_ID,
    "notification_uuid": FAKE_UUID,
    "created_at": "2018-03-22T00:00:00.000000",
    "updated_at": "2018-03-23T00:00:00.000000",
    "type": "pacemaker",
    "hostname": "fake_host",
    "status": "new",
    "generated_time": "2018-03-21T00:00:00.000000",
    "payload": PAYLOAD,
    "source_host_uuid": FAKE_HOST_UUID,
    "recovery_workflow_details": RECOVERY_WORKFLOW_DETAILS
}


class TestNotification(base.TestCase):

    def test_basic(self):
        sot = notification.Notification(NOTIFICATION)
        self.assertEqual("notification", sot.resource_key)
        self.assertEqual("notifications", sot.resources_key)
        self.assertEqual("/notifications", sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)

        self.assertDictEqual({"generated_since": "generated-since",
                              "limit": "limit",
                              "marker": "marker",
                              "sort_dir": "sort_dir",
                              "sort_key": "sort_key",
                              "source_host_uuid": "source_host_uuid",
                              "status": "status",
                              "type": "type"},
                             sot._query_mapping._mapping)

    def test_create(self):
        sot = notification.Notification(**NOTIFICATION)
        rec_workflow_details = NOTIFICATION["recovery_workflow_details"][0]
        self.assertEqual(NOTIFICATION["id"], sot.id)
        self.assertEqual(
            NOTIFICATION["notification_uuid"], sot.notification_uuid)
        self.assertEqual(NOTIFICATION["created_at"], sot.created_at)
        self.assertEqual(NOTIFICATION["updated_at"], sot.updated_at)
        self.assertEqual(NOTIFICATION["type"], sot.type)
        self.assertEqual(NOTIFICATION["hostname"], sot.hostname)
        self.assertEqual(NOTIFICATION["status"], sot.status)
        self.assertEqual(NOTIFICATION["generated_time"], sot.generated_time)
        self.assertEqual(NOTIFICATION["payload"], sot.payload)
        self.assertEqual(
            NOTIFICATION["source_host_uuid"], sot.source_host_uuid)
        self.assertEqual(rec_workflow_details["name"],
                         sot.recovery_workflow_details[0].name)
        self.assertEqual(rec_workflow_details["state"],
                         sot.recovery_workflow_details[0].state)
        self.assertEqual(rec_workflow_details["progress"],
                         sot.recovery_workflow_details[0].progress)
        self.assertEqual(
            rec_workflow_details["progress_details"][0]['progress'],
            sot.recovery_workflow_details[0].progress_details[0].progress)
        self.assertEqual(
            rec_workflow_details["progress_details"][0]['message'],
            sot.recovery_workflow_details[0].progress_details[0].message)
        self.assertEqual(
            rec_workflow_details["progress_details"][0]['timestamp'],
            sot.recovery_workflow_details[0].progress_details[0].timestamp)
        self.assertIsInstance(sot.recovery_workflow_details, list)
        self.assertIsInstance(
            sot.recovery_workflow_details[0].progress_details, list)
        self.assertIsInstance(sot.recovery_workflow_details[0],
                              notification.RecoveryWorkflowDetailItem)
        self.assertIsInstance(
            sot.recovery_workflow_details[0].progress_details[0],
            notification.ProgressDetailsItem)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.accelerator.v2 import accelerator_request as _arq
from openstack.accelerator.v2 import deployable as _deployable
from openstack.accelerator.v2 import device as _device
from openstack.accelerator.v2 import device_profile as _device_profile
from openstack import proxy


class Proxy(proxy.Proxy):

    def deployables(self, **query):
        """Retrieve a generator of deployables.

        :param kwargs query: Optional query parameters to be sent to
            restrict the deployables to be returned.
        :returns: A generator of deployable instances.
        """
        return self._list(_deployable.Deployable, **query)

    def get_deployable(self, uuid, fields=None):
        """Get a single deployable.

        :param uuid: The value can be the UUID of a deployable.
        :returns: One :class:`~openstack.accelerator.v2.deployable.Deployable`
        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
            deployable matching the criteria could be found.
        """
        return self._get(_deployable.Deployable, uuid)

    def update_deployable(self, uuid, patch):
        """Reconfig the FPGA with new bitstream.

        :param uuid: The value can be the UUID of a deployable
        :param patch: The information to reconfig.
        :returns: The results of FPGA reconfig.
        """
        return self._get_resource(_deployable.Deployable,
                                  uuid).patch(self, patch)

    def devices(self, **query):
        """Retrieve a generator of devices.

        :param kwargs query: Optional query parameters to be sent to
            restrict the devices to be returned. Available parameters include:

            * hostname: The hostname of the device.
            * type: The type of the device.
            * vendor: The vendor ID of the device.
            * sort: A list of sorting keys separated by commas. Each sorting
              key can optionally be attached with a sorting direction
              modifier which can be ``asc`` or ``desc``.
            * limit: Requests a specified size of returned items from the
              query. Returns a number of items up to the specified limit
              value.
            * marker: Specifies the ID of the last-seen item. Use the limit
              parameter to make an initial limited request and use the ID of
              the last-seen item from the response as the marker parameter
              value in a subsequent limited request.
        :returns: A generator of device instances.
        """
        return self._list(_device.Device, **query)

    def get_device(self, uuid, fields=None):
        """Get a single device.

        :param uuid: The value can be the UUID of a device.
        :returns: One :class:`~openstack.accelerator.v2.device.Device`
        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
            device matching the criteria could be found.
        """
        return self._get(_device.Device, uuid)

    def device_profiles(self, **query):
        """Retrieve a generator of device profiles.

        :param kwargs query: Optional query parameters to be sent to
            restrict the device profiles to be returned.
        :returns: A generator of device profile instances.
        """
        return self._list(_device_profile.DeviceProfile, **query)

    def create_device_profile(self, **attrs):
        """Create a device_profile.

        :param kwargs attrs: a list of device_profiles.
        :returns: The list of created device profiles
        """
        return self._create(_device_profile.DeviceProfile, **attrs)

    def delete_device_profile(self, device_profile, ignore_missing=True):
        """Delete a device profile

        :param device_profile: The value can be either the ID of a device
            profile or a
            :class:`~openstack.accelerator.v2.device_profile.DeviceProfile`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the device profile does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent device profile.
        :returns: ``None``
        """
        return self._delete(
            _device_profile.DeviceProfile,
            device_profile,
            ignore_missing=ignore_missing)

    def get_device_profile(self, uuid, fields=None):
        """Get a single device profile.

        :param uuid: The value can be the UUID of a device profile.
        :returns: One :class:
            `~openstack.accelerator.v2.device_profile.DeviceProfile`
        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
            device profile matching the criteria could be found.
        """
        return self._get(_device_profile.DeviceProfile, uuid)

    def accelerator_requests(self, **query):
        """Retrieve a generator of accelerator requests.

        :param kwargs query: Optional query parameters to be sent to
            restrict the accelerator requests to be returned.
        :returns: A generator of accelerator request instances.
        """
        return self._list(_arq.AcceleratorRequest, **query)

    def create_accelerator_request(self, **attrs):
        """Create an ARQs for a single device profile.

        :param kwargs attrs: request body.
        :returns: The created accelerator request instance.
        """
        return self._create(_arq.AcceleratorRequest, **attrs)

    def delete_accelerator_request(
        self, accelerator_request, ignore_missing=True,
    ):
        """Delete a device profile

        :param device_profile: The value can be either the ID of a device
            profile or a
            :class:`~openstack.accelerator.v2.device_profile.DeviceProfile`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the device profile does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent accelerator request.
        :returns: ``None``
        """
        return self._delete(
            _arq.AcceleratorRequest,
            accelerator_request,
            ignore_missing=ignore_missing)

    def get_accelerator_request(self, uuid, fields=None):
        """Get a single accelerator request.

        :param uuid: The value can be the UUID of a accelerator request.
        :returns: One :class:
            `~openstack.accelerator.v2.accelerator_request.AcceleratorRequest`
        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
            accelerator request matching the criteria could be found.
        """
        return self._get(_arq.AcceleratorRequest, uuid)

    def update_accelerator_request(self, uuid, properties):
        """Bind/Unbind an accelerator to VM.

        :param uuid: The uuid of the accelerator_request to be bound/unbound.
        :param properties: The info of VM
            that will bind/unbind the accelerator.
        :returns: True if bind/unbind succeeded, False otherwise.
        """
        return self._get_resource(_arq.AcceleratorRequest,
                                  uuid).patch(self, properties)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import tempfile

import testtools

import openstack.cloud
from openstack.orchestration.v1 import stack
from openstack.tests import fakes
from openstack.tests.unit import base


class TestStack(base.TestCase):

    def setUp(self):
        super(TestStack, self).setUp()
        self.stack_id = self.getUniqueString('id')
        self.stack_name = self.getUniqueString('name')
        self.stack_tag = self.getUniqueString('tag')
        self.stack = fakes.make_fake_stack(self.stack_id, self.stack_name)

    def _compare_stacks(self, exp, real):
        self.assertDictEqual(
            stack.Stack(**exp).to_dict(computed=False),
            real.to_dict(computed=False))

    def test_list_stacks(self):
        fake_stacks = [
            self.stack,
            fakes.make_fake_stack(
                self.getUniqueString('id'),
                self.getUniqueString('name'))
        ]
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/stacks'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT),
                 json={"stacks": fake_stacks}),
        ])
        stacks = self.cloud.list_stacks()
        [self._compare_stacks(b, a) for a, b in zip(stacks, fake_stacks)]

        self.assert_calls()

    def test_list_stacks_filters(self):
        fake_stacks = [
            self.stack,
            fakes.make_fake_stack(
                self.getUniqueString('id'),
                self.getUniqueString('name'))
        ]
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'orchestration', 'public',
                     append=['stacks'],
                     qs_elements=['name=a', 'status=b'],
                 ),
                 json={"stacks": fake_stacks}),
        ])
        stacks = self.cloud.list_stacks(name='a', status='b')
        [self._compare_stacks(b, a) for a, b in zip(stacks, fake_stacks)]

        self.assert_calls()

    def test_list_stacks_exception(self):
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/stacks'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT),
                 status_code=404)
        ])
        with testtools.ExpectedException(
                openstack.cloud.OpenStackCloudURINotFound):
            self.cloud.list_stacks()
        self.assert_calls()

    def test_search_stacks(self):
        fake_stacks = [
            self.stack,
            fakes.make_fake_stack(
                self.getUniqueString('id'),
                self.getUniqueString('name'))
        ]
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/stacks'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT),
                 json={"stacks": fake_stacks}),
        ])
        stacks = self.cloud.search_stacks()
        [self._compare_stacks(b, a) for a, b in zip(stacks, fake_stacks)]
        self.assert_calls()

    def test_search_stacks_filters(self):
        fake_stacks = [
            self.stack,
            fakes.make_fake_stack(
                self.getUniqueString('id'),
                self.getUniqueString('name'),
                status='CREATE_FAILED')
        ]
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/stacks'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT),
                 json={"stacks": fake_stacks}),
        ])
        filters = {'status': 'FAILED'}
        stacks = self.cloud.search_stacks(filters=filters)
        [self._compare_stacks(b, a) for a, b in zip(stacks, fake_stacks)]
        self.assert_calls()

    def test_search_stacks_exception(self):
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/stacks'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT),
                 status_code=404)
        ])
        with testtools.ExpectedException(
                openstack.cloud.OpenStackCloudURINotFound):
            self.cloud.search_stacks()

    def test_delete_stack(self):
        resolve = 'resolve_outputs=False'
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/stacks/{name}?{resolve}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     name=self.stack_name, resolve=resolve),
                 status_code=302,
                 headers=dict(
                     location='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                         endpoint=fakes.ORCHESTRATION_ENDPOINT,
                         id=self.stack_id, name=self.stack_name,
                         resolve=resolve))),
            dict(method='GET',
                 uri='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id, name=self.stack_name, resolve=resolve),
                 json={"stack": self.stack}),
            dict(method='DELETE',
                 uri='{endpoint}/stacks/{id}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id)),
        ])
        self.assertTrue(self.cloud.delete_stack(self.stack_name))
        self.assert_calls()

    def test_delete_stack_not_found(self):
        resolve = 'resolve_outputs=False'
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/stacks/stack_name?{resolve}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT, resolve=resolve),
                 status_code=404),
        ])
        self.assertFalse(self.cloud.delete_stack('stack_name'))
        self.assert_calls()

    def test_delete_stack_exception(self):
        resolve = 'resolve_outputs=False'
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/stacks/{id}?{resolve}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id, resolve=resolve),
                 status_code=302,
                 headers=dict(
                     location='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                         endpoint=fakes.ORCHESTRATION_ENDPOINT,
                         id=self.stack_id, name=self.stack_name,
                         resolve=resolve))),
            dict(method='GET',
                 uri='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id, name=self.stack_name, resolve=resolve),
                 json={"stack": self.stack}),
            dict(method='DELETE',
                 uri='{endpoint}/stacks/{id}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id),
                 status_code=400,
                 reason="ouch"),
        ])
        with testtools.ExpectedException(
                openstack.cloud.OpenStackCloudBadRequest):
            self.cloud.delete_stack(self.stack_id)
        self.assert_calls()

    def test_delete_stack_by_name_wait(self):
        marker_event = fakes.make_fake_stack_event(
            self.stack_id, self.stack_name, status='CREATE_COMPLETE',
            resource_name='name')
        marker_qs = 'marker={e_id}&sort_dir=asc'.format(
            e_id=marker_event['id'])
        resolve = 'resolve_outputs=False'
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/stacks/{name}?{resolve}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     name=self.stack_name,
                     resolve=resolve),
                 status_code=302,
                 headers=dict(
                     location='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                         endpoint=fakes.ORCHESTRATION_ENDPOINT,
                         id=self.stack_id, name=self.stack_name,
                         resolve=resolve))),
            dict(method='GET',
                 uri='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id, name=self.stack_name, resolve=resolve),
                 json={"stack": self.stack}),
            dict(method='GET',
                 uri='{endpoint}/stacks/{name}/events?{qs}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     name=self.stack_name,
                     qs='limit=1&sort_dir=desc'),
                 complete_qs=True,
                 json={"events": [marker_event]}),
            dict(method='DELETE',
                 uri='{endpoint}/stacks/{id}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id)),
            dict(method='GET',
                 uri='{endpoint}/stacks/{name}/events?{qs}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     name=self.stack_name,
                     qs=marker_qs),
                 complete_qs=True,
                 json={"events": [
                     fakes.make_fake_stack_event(
                         self.stack_id, self.stack_name,
                         status='DELETE_COMPLETE', resource_name='name'),
                 ]}),
            dict(method='GET',
                 uri='{endpoint}/stacks/{name}?{resolve}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     name=self.stack_name, resolve=resolve),
                 status_code=404),
        ])

        self.assertTrue(self.cloud.delete_stack(self.stack_name, wait=True))
        self.assert_calls()

    def test_delete_stack_by_id_wait(self):
        marker_event = fakes.make_fake_stack_event(
            self.stack_id, self.stack_name, status='CREATE_COMPLETE',
            resource_name='name')
        marker_qs = 'marker={e_id}&sort_dir=asc'.format(
            e_id=marker_event['id'])
        resolve = 'resolve_outputs=False'
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/stacks/{id}?{resolve}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id,
                     resolve=resolve),
                 status_code=302,
                 headers=dict(
                     location='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                         endpoint=fakes.ORCHESTRATION_ENDPOINT,
                         id=self.stack_id, name=self.stack_name,
                         resolve=resolve))),
            dict(method='GET',
                 uri='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id, name=self.stack_name, resolve=resolve),
                 json={"stack": self.stack}),
            dict(method='GET',
                 uri='{endpoint}/stacks/{id}/events?{qs}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id,
                     qs='limit=1&sort_dir=desc'),
                 complete_qs=True,
                 json={"events": [marker_event]}),
            dict(method='DELETE',
                 uri='{endpoint}/stacks/{id}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id)),
            dict(method='GET',
                 uri='{endpoint}/stacks/{id}/events?{qs}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id,
                     qs=marker_qs),
                 complete_qs=True,
                 json={"events": [
                     fakes.make_fake_stack_event(
                         self.stack_id, self.stack_name,
                         status='DELETE_COMPLETE'),
                 ]}),
            dict(method='GET',
                 uri='{endpoint}/stacks/{id}?{resolve}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id, resolve=resolve),
                 status_code=404),
        ])

        self.assertTrue(self.cloud.delete_stack(self.stack_id, wait=True))
        self.assert_calls()

    def test_delete_stack_wait_failed(self):
        failed_stack = self.stack.copy()
        failed_stack['stack_status'] = 'DELETE_FAILED'
        marker_event = fakes.make_fake_stack_event(
            self.stack_id, self.stack_name, status='CREATE_COMPLETE')
        marker_qs = 'marker={e_id}&sort_dir=asc'.format(
            e_id=marker_event['id'])
        resolve = 'resolve_outputs=False'
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/stacks/{id}?{resolve}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id, resolve=resolve),
                 status_code=302,
                 headers=dict(
                     location='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                         endpoint=fakes.ORCHESTRATION_ENDPOINT,
                         id=self.stack_id, name=self.stack_name,
                         resolve=resolve))),
            dict(method='GET',
                 uri='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id, name=self.stack_name, resolve=resolve),
                 json={"stack": self.stack}),
            dict(method='GET',
                 uri='{endpoint}/stacks/{id}/events?{qs}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id,
                     qs='limit=1&sort_dir=desc'),
                 complete_qs=True,
                 json={"events": [marker_event]}),
            dict(method='DELETE',
                 uri='{endpoint}/stacks/{id}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id)),
            dict(method='GET',
                 uri='{endpoint}/stacks/{id}/events?{qs}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id,
                     qs=marker_qs),
                 complete_qs=True,
                 json={"events": [
                     fakes.make_fake_stack_event(
                         self.stack_id, self.stack_name,
                         status='DELETE_COMPLETE'),
                 ]}),
            dict(method='GET',
                 uri='{endpoint}/stacks/{id}?resolve_outputs=False'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id),
                 status_code=302,
                 headers=dict(
                     location='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                         endpoint=fakes.ORCHESTRATION_ENDPOINT,
                         id=self.stack_id, name=self.stack_name,
                         resolve=resolve))),
            dict(method='GET',
                 uri='{endpoint}/stacks/{name}/{id}?{resolve}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id, name=self.stack_name, resolve=resolve),
                 json={"stack": failed_stack}),
        ])

        with testtools.ExpectedException(
                openstack.cloud.OpenStackCloudException):
            self.cloud.delete_stack(self.stack_id, wait=True)

        self.assert_calls()

    def test_create_stack(self):
        test_template = tempfile.NamedTemporaryFile(delete=False)
        test_template.write(fakes.FAKE_TEMPLATE.encode('utf-8'))
        test_template.close()
        self.register_uris([
            dict(
                method='POST', uri='{endpoint}/stacks'.format(
                    endpoint=fakes.ORCHESTRATION_ENDPOINT),
                json={"stack": self.stack},
                validate=dict(
                    json={
                        'disable_rollback': False,
                        'parameters': {},
                        'stack_name': self.stack_name,
                        'tags': self.stack_tag,
                        'template': fakes.FAKE_TEMPLATE_CONTENT,
                        'timeout_mins': 60}
                )),
            dict(
                method='GET',
                uri='{endpoint}/stacks/{name}'.format(
                    endpoint=fakes.ORCHESTRATION_ENDPOINT,
                    name=self.stack_name),
                status_code=302,
                headers=dict(
                    location='{endpoint}/stacks/{name}/{id}'.format(
                        endpoint=fakes.ORCHESTRATION_ENDPOINT,
                        id=self.stack_id, name=self.stack_name))),
            dict(
                method='GET',
                uri='{endpoint}/stacks/{name}/{id}'.format(
                    endpoint=fakes.ORCHESTRATION_ENDPOINT,
                    id=self.stack_id, name=self.stack_name),
                json={"stack": self.stack}),
        ])

        self.cloud.create_stack(
            self.stack_name,
            tags=self.stack_tag,
            template_file=test_template.name
        )

        self.assert_calls()

    def test_create_stack_wait(self):

        test_template = tempfile.NamedTemporaryFile(delete=False)
        test_template.write(fakes.FAKE_TEMPLATE.encode('utf-8'))
        test_template.close()

        self.register_uris([
            dict(
                method='POST', uri='{endpoint}/stacks'.format(
                    endpoint=fakes.ORCHESTRATION_ENDPOINT),
                json={"stack": self.stack},
                validate=dict(
                    json={
                        'disable_rollback': False,
                        'parameters': {},
                        'stack_name': self.stack_name,
                        'tags': self.stack_tag,
                        'template': fakes.FAKE_TEMPLATE_CONTENT,
                        'timeout_mins': 60}
                )),
            dict(
                method='GET',
                uri='{endpoint}/stacks/{name}/events?sort_dir=asc'.format(
                    endpoint=fakes.ORCHESTRATION_ENDPOINT,
                    name=self.stack_name),
                json={"events": [
                    fakes.make_fake_stack_event(
                        self.stack_id, self.stack_name,
                        status='CREATE_COMPLETE',
                        resource_name='name'),
                ]}),
            dict(
                method='GET',
                uri='{endpoint}/stacks/{name}'.format(
                    endpoint=fakes.ORCHESTRATION_ENDPOINT,
                    name=self.stack_name),
                status_code=302,
                headers=dict(
                    location='{endpoint}/stacks/{name}/{id}'.format(
                        endpoint=fakes.ORCHESTRATION_ENDPOINT,
                        id=self.stack_id, name=self.stack_name))),
            dict(
                method='GET',
                uri='{endpoint}/stacks/{name}/{id}'.format(
                    endpoint=fakes.ORCHESTRATION_ENDPOINT,
                    id=self.stack_id, name=self.stack_name),
                json={"stack": self.stack}),
        ])
        self.cloud.create_stack(
            self.stack_name,
            tags=self.stack_tag,
            template_file=test_template.name,
            wait=True)

        self.assert_calls()

    def test_update_stack(self):
        test_template = tempfile.NamedTemporaryFile(delete=False)
        test_template.write(fakes.FAKE_TEMPLATE.encode('utf-8'))
        test_template.close()

        self.register_uris([
            dict(
                method='PUT',
                uri='{endpoint}/stacks/{name}'.format(
                    endpoint=fakes.ORCHESTRATION_ENDPOINT,
                    name=self.stack_name),
                validate=dict(
                    json={
                        'disable_rollback': False,
                        'parameters': {},
                        'tags': self.stack_tag,
                        'template': fakes.FAKE_TEMPLATE_CONTENT,
                        'timeout_mins': 60}),
                json={}),
            dict(
                method='GET',
                uri='{endpoint}/stacks/{name}'.format(
                    endpoint=fakes.ORCHESTRATION_ENDPOINT,
                    name=self.stack_name),
                status_code=302,
                headers=dict(
                    location='{endpoint}/stacks/{name}/{id}'.format(
                        endpoint=fakes.ORCHESTRATION_ENDPOINT,
                        id=self.stack_id, name=self.stack_name))),
            dict(
                method='GET',
                uri='{endpoint}/stacks/{name}/{id}'.format(
                    endpoint=fakes.ORCHESTRATION_ENDPOINT,
                    id=self.stack_id, name=self.stack_name),
                json={"stack": self.stack}),
        ])
        self.cloud.update_stack(
            self.stack_name,
            tags=self.stack_tag,
            template_file=test_template.name)

        self.assert_calls()

    def test_update_stack_wait(self):
        marker_event = fakes.make_fake_stack_event(
            self.stack_id, self.stack_name, status='CREATE_COMPLETE',
            resource_name='name')
        marker_qs = 'marker={e_id}&sort_dir=asc'.format(
            e_id=marker_event['id'])
        test_template = tempfile.NamedTemporaryFile(delete=False)
        test_template.write(fakes.FAKE_TEMPLATE.encode('utf-8'))
        test_template.close()

        self.register_uris([
            dict(
                method='GET',
                uri='{endpoint}/stacks/{name}/events?{qs}'.format(
                    endpoint=fakes.ORCHESTRATION_ENDPOINT,
                    name=self.stack_name,
                    qs='limit=1&sort_dir=desc'),
                json={"events": [marker_event]}),
            dict(
                method='PUT',
                uri='{endpoint}/stacks/{name}'.format(
                    endpoint=fakes.ORCHESTRATION_ENDPOINT,
                    name=self.stack_name),
                validate=dict(
                    json={
                        'disable_rollback': False,
                        'parameters': {},
                        'tags': self.stack_tag,
                        'template': fakes.FAKE_TEMPLATE_CONTENT,
                        'timeout_mins': 60}),
                json={}),
            dict(
                method='GET',
                uri='{endpoint}/stacks/{name}/events?{qs}'.format(
                    endpoint=fakes.ORCHESTRATION_ENDPOINT,
                    name=self.stack_name,
                    qs=marker_qs),
                json={"events": [
                    fakes.make_fake_stack_event(
                        self.stack_id, self.stack_name,
                        status='UPDATE_COMPLETE',
                        resource_name='name'),
                ]}),
            dict(
                method='GET',
                uri='{endpoint}/stacks/{name}'.format(
                    endpoint=fakes.ORCHESTRATION_ENDPOINT,
                    name=self.stack_name),
                status_code=302,
                headers=dict(
                    location='{endpoint}/stacks/{name}/{id}'.format(
                        endpoint=fakes.ORCHESTRATION_ENDPOINT,
                        id=self.stack_id, name=self.stack_name))),
            dict(
                method='GET',
                uri='{endpoint}/stacks/{name}/{id}'.format(
                    endpoint=fakes.ORCHESTRATION_ENDPOINT,
                    id=self.stack_id, name=self.stack_name),
                json={"stack": self.stack}),
        ])
        self.cloud.update_stack(
            self.stack_name,
            tags=self.stack_tag,
            template_file=test_template.name,
            wait=True)

        self.assert_calls()

    def test_get_stack(self):
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/stacks/{name}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     name=self.stack_name),
                 status_code=302,
                 headers=dict(
                     location='{endpoint}/stacks/{name}/{id}'.format(
                         endpoint=fakes.ORCHESTRATION_ENDPOINT,
                         id=self.stack_id, name=self.stack_name))),
            dict(method='GET',
                 uri='{endpoint}/stacks/{name}/{id}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id, name=self.stack_name),
                 json={"stack": self.stack}),
        ])

        res = self.cloud.get_stack(self.stack_name)
        self.assertIsNotNone(res)
        self.assertEqual(self.stack['stack_name'], res['name'])
        self.assertEqual(self.stack['stack_status'], res['stack_status'])
        self.assertEqual('CREATE_COMPLETE', res['status'])

        self.assert_calls()

    def test_get_stack_in_progress(self):
        in_progress = self.stack.copy()
        in_progress['stack_status'] = 'CREATE_IN_PROGRESS'
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/stacks/{name}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     name=self.stack_name),
                 status_code=302,
                 headers=dict(
                     location='{endpoint}/stacks/{name}/{id}'.format(
                         endpoint=fakes.ORCHESTRATION_ENDPOINT,
                         id=self.stack_id, name=self.stack_name))),
            dict(method='GET',
                 uri='{endpoint}/stacks/{name}/{id}'.format(
                     endpoint=fakes.ORCHESTRATION_ENDPOINT,
                     id=self.stack_id, name=self.stack_name),
                 json={"stack": in_progress}),
        ])

        res = self.cloud.get_stack(self.stack_name)
        self.assertIsNotNone(res)
        self.assertEqual(in_progress['stack_name'], res.name)
        self.assertEqual(in_progress['stack_status'], res['stack_status'])
        self.assertEqual('CREATE_IN_PROGRESS', res['status'])

        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import metering_label
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'description': '1',
    'id': IDENTIFIER,
    'name': '3',
    'project_id': '4',
    'shared': False,
}


class TestMeteringLabel(base.TestCase):

    def test_basic(self):
        sot = metering_label.MeteringLabel()
        self.assertEqual('metering_label', sot.resource_key)
        self.assertEqual('metering_labels', sot.resources_key)
        self.assertEqual('/metering/metering-labels', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = metering_label.MeteringLabel(**EXAMPLE)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['shared'], sot.is_shared)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class Role(resource.Resource):
    resource_key = 'role'
    resources_key = 'roles'
    base_path = '/roles'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True
    commit_method = 'PATCH'

    _query_mapping = resource.QueryParameters(
        'name', 'domain_id')

    # Properties
    #: Unique role name, within the owning domain. *Type: string*
    name = resource.Body('name')
    #: User-facing description of the role. *Type: string*
    description = resource.Body('description')
    #: References the domain ID which owns the role. *Type: string*
    domain_id = resource.Body('domain_id')
    #: The links for the service resource.
    links = resource.Body('links')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import exceptions
from openstack import resource
from openstack import utils


class Type(resource.Resource):
    resource_key = "volume_type"
    resources_key = "volume_types"
    base_path = "/types"

    # capabilities
    allow_fetch = True
    allow_create = True
    allow_delete = True
    allow_list = True
    allow_commit = True

    _query_mapping = resource.QueryParameters("is_public")

    # Properties
    #: Description of the type.
    description = resource.Body("description")
    #: A dict of extra specifications. "capabilities" is a usual key.
    extra_specs = resource.Body("extra_specs", type=dict)
    #: a private volume-type. *Type: bool*
    is_public = resource.Body('os-volume-type-access:is_public', type=bool)

    def _extra_specs(self, method, key=None, delete=False,
                     extra_specs=None):
        extra_specs = extra_specs or {}
        for k, v in extra_specs.items():
            if not isinstance(v, str):
                raise ValueError("The value for %s (%s) must be "
                                 "a text string" % (k, v))

        if key is not None:
            url = utils.urljoin(self.base_path, self.id, "extra_specs", key)
        else:
            url = utils.urljoin(self.base_path, self.id, "extra_specs")

        kwargs = {}
        if extra_specs:
            kwargs["json"] = {"extra_specs": extra_specs}

        response = method(url, headers={}, **kwargs)

        # ensure Cinder API has not returned us an error
        exceptions.raise_from_response(response)
        # DELETE doesn't return a JSON body while everything else does.
        return response.json() if not delete else None

    def set_extra_specs(self, session, **extra_specs):
        """Update extra_specs

        This call will replace only the extra_specs with the same keys
        given here.  Other keys will not be modified.

        :param session: The session to use for this request.
        :param kwargs extra_specs: key/value extra_specs pairs to be update on
                                   this volume type. All keys and values
        """
        if not extra_specs:
            return dict()

        result = self._extra_specs(session.post, extra_specs=extra_specs)
        return result["extra_specs"]

    def delete_extra_specs(self, session, keys):
        """Delete extra_specs

        Note: This method will do a HTTP DELETE request for every key in keys.

        :param session: The session to use for this request.
        :param list keys: The keys to delete.

        :rtype: ``None``
        """
        for key in keys:
            self._extra_specs(session.delete, key=key, delete=True)

    def get_private_access(self, session):
        url = utils.urljoin(self.base_path, self.id, "os-volume-type-access")
        resp = session.get(url)

        exceptions.raise_from_response(resp)

        return resp.json().get("volume_type_access", [])

    def add_private_access(self, session, project_id):
        url = utils.urljoin(self.base_path, self.id, "action")
        body = {"addProjectAccess": {"project": project_id}}

        resp = session.post(url, json=body)

        exceptions.raise_from_response(resp)

    def remove_private_access(self, session, project_id):
        url = utils.urljoin(self.base_path, self.id, "action")
        body = {"removeProjectAccess": {"project": project_id}}

        resp = session.post(url, json=body)

        exceptions.raise_from_response(resp)


class TypeEncryption(resource.Resource):
    resource_key = "encryption"
    resources_key = "encryption"
    base_path = "/types/%(volume_type_id)s/encryption"

    # capabilities
    allow_fetch = True
    allow_create = True
    allow_delete = True
    allow_list = False
    allow_commit = True

    # Properties
    #: The encryption algorithm or mode.
    cipher = resource.Body("cipher")
    #: Notional service where encryption is performed.
    control_location = resource.Body("control_location")
    #: The date and time when the resource was created.
    created_at = resource.Body("created_at")
    #: The resource is deleted or not.
    deleted = resource.Body("deleted")
    #: The date and time when the resource was deleted.
    deleted_at = resource.Body("deleted_at")
    #: A ID representing this type.
    encryption_id = resource.Body("encryption_id", alternate_id=True)
    #: The Size of encryption key.
    key_size = resource.Body("key_size")
    #: The class that provides encryption support.
    provider = resource.Body("provider")
    #: The date and time when the resource was updated.
    updated_at = resource.Body("updated_at")
    #: The ID of the Volume Type.
    volume_type_id = resource.URI("volume_type_id")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.common import tag
from openstack import exceptions
from openstack import resource
from openstack import utils


class Stack(resource.Resource):
    name_attribute = 'stack_name'
    resource_key = 'stack'
    resources_key = 'stacks'
    base_path = '/stacks'

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True

    _query_mapping = resource.QueryParameters(
        'action', 'name', 'status',
        'project_id', 'owner_id', 'username',
        project_id='tenant_id',
        **tag.TagMixin._tag_query_parameters
    )

    # Properties
    #: A list of resource objects that will be added if a stack update
    #  is performed.
    added = resource.Body('added')
    #: Placeholder for AWS compatible template listing capabilities
    #: required by the stack.
    capabilities = resource.Body('capabilities')
    #: Timestamp of the stack creation.
    created_at = resource.Body('creation_time')
    #: A text description of the stack.
    description = resource.Body('description')
    #: A list of resource objects that will be deleted if a stack
    #: update is performed.
    deleted = resource.Body('deleted', type=list)
    #: Timestamp of the stack deletion.
    deleted_at = resource.Body('deletion_time')
    #: A JSON environment for the stack.
    environment = resource.Body('environment')
    #: An ordered list of names for environment files found in the files dict.
    environment_files = resource.Body('environment_files', type=list)
    #: Additional files referenced in the template or the environment
    files = resource.Body('files', type=dict)
    #: Name of the container in swift that has child
    #: templates and environment files.
    files_container = resource.Body('files_container')
    #: Whether the stack will support a rollback operation on stack
    #: create/update failures. *Type: bool*
    is_rollback_disabled = resource.Body('disable_rollback', type=bool)
    #: A list of dictionaries containing links relevant to the stack.
    links = resource.Body('links')
    #: Name of the stack.
    name = resource.Body('stack_name')
    stack_name = resource.URI('stack_name')
    #: Placeholder for future extensions where stack related events
    #: can be published.
    notification_topics = resource.Body('notification_topics')
    #: A list containing output keys and values from the stack, if any.
    outputs = resource.Body('outputs')
    #: The ID of the owner stack if any.
    owner_id = resource.Body('stack_owner')
    #: A dictionary containing the parameter names and values for the stack.
    parameters = resource.Body('parameters', type=dict)
    #: The ID of the parent stack if any
    parent_id = resource.Body('parent')
    #: A list of resource objects that will be replaced if a stack update
    #: is performed.
    replaced = resource.Body('replaced')
    #: A string representation of the stack status, e.g. ``CREATE_COMPLETE``.
    status = resource.Body('stack_status')
    #: A text explaining how the stack transits to its current status.
    status_reason = resource.Body('stack_status_reason')
    #: A list of strings used as tags on the stack
    tags = resource.Body('tags', type=list, default=[])
    #: A dict containing the template use for stack creation.
    template = resource.Body('template', type=dict)
    #: Stack template description text. Currently contains the same text
    #: as that of the ``description`` property.
    template_description = resource.Body('template_description')
    #: A string containing the URL where a stack template can be found.
    template_url = resource.Body('template_url')
    #: Stack operation timeout in minutes.
    timeout_mins = resource.Body('timeout_mins')
    #: A list of resource objects that will remain unchanged if a stack
    #: update is performed.
    unchanged = resource.Body('unchanged')
    #: A list of resource objects that will have their properties updated
    #: in place if a stack update is performed.
    updated = resource.Body('updated')
    #: Timestamp of last update on the stack.
    updated_at = resource.Body('updated_time')
    #: The ID of the user project created for this stack.
    user_project_id = resource.Body('stack_user_project_id')

    def create(self, session, base_path=None):
        # This overrides the default behavior of resource creation because
        # heat doesn't accept resource_key in its request.
        return super(Stack, self).create(session, prepend_key=False,
                                         base_path=base_path)

    def commit(self, session, base_path=None):
        # This overrides the default behavior of resource creation because
        # heat doesn't accept resource_key in its request.
        return super(Stack, self).commit(session, prepend_key=False,
                                         has_body=False, base_path=None)

    def update(self, session, preview=False):
        # This overrides the default behavior of resource update because
        # we need to use other endpoint for update preview.
        base_path = None
        if self.name and self.id:
            base_path = '/stacks/%(stack_name)s/%(stack_id)s' % {
                'stack_name': self.name,
                'stack_id': self.id}
        elif self.name or self.id:
            # We have only one of name/id. Do not try to build a stacks/NAME/ID
            # path
            base_path = '/stacks/%(stack_identity)s' % {
                'stack_identity': self.name or self.id}
        request = self._prepare_request(
            prepend_key=False,
            requires_id=False,
            base_path=base_path)

        microversion = self._get_microversion(session, action='commit')

        request_url = request.url
        if preview:
            request_url = utils.urljoin(request_url, 'preview')

        response = session.put(
            request_url, json=request.body, headers=request.headers,
            microversion=microversion)

        self.microversion = microversion
        self._translate_response(response, has_body=True)
        return self

    def _action(self, session, body):
        """Perform stack actions"""
        url = utils.urljoin(self.base_path, self._get_id(self), 'actions')
        resp = session.post(url, json=body)
        return resp.json()

    def check(self, session):
        return self._action(session, {'check': ''})

    def abandon(self, session):
        url = utils.urljoin(self.base_path, self.name,
                            self._get_id(self), 'abandon')
        resp = session.delete(url)
        return resp.json()

    def fetch(self, session, requires_id=True,
              base_path=None, error_message=None,
              skip_cache=False, resolve_outputs=True):

        if not self.allow_fetch:
            raise exceptions.MethodNotSupported(self, "fetch")

        request = self._prepare_request(requires_id=requires_id,
                                        base_path=base_path)
        # session = self._get_session(session)
        microversion = self._get_microversion(session, action='fetch')

        # NOTE(gtema): would be nice to simply use QueryParameters, however
        # Heat return 302 with parameters being set into URL and requests
        # apply parameters again, what results in them being set doubled
        if not resolve_outputs:
            request.url = request.url + '?resolve_outputs=False'
        response = session.get(
            request.url, microversion=microversion, skip_cache=skip_cache)
        kwargs = {}
        if error_message:
            kwargs['error_message'] = error_message

        self.microversion = microversion
        self._translate_response(response, **kwargs)

        if self and self.status in ['DELETE_COMPLETE', 'ADOPT_COMPLETE']:
            raise exceptions.ResourceNotFound(
                "No stack found for %s" % self.id)
        return self

    @classmethod
    def find(cls, session, name_or_id, ignore_missing=True, **params):
        """Find a resource by its name or id.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param name_or_id: This resource's identifier, if needed by
                           the request. The default is ``None``.
        :param bool ignore_missing: When set to ``False``
                    :class:`~openstack.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :param dict params: Any additional parameters to be passed into
                            underlying methods, such as to
                            :meth:`~openstack.resource.Resource.existing`
                            in order to pass on URI parameters.

        :return: The :class:`Resource` object matching the given name or id
                 or None if nothing matches.
        :raises: :class:`openstack.exceptions.DuplicateResource` if more
                 than one resource is found for this request.
        :raises: :class:`openstack.exceptions.ResourceNotFound` if nothing
                 is found and ignore_missing is ``False``.
        """
        session = cls._get_session(session)
        # Try to short-circuit by looking directly for a matching ID.
        try:
            match = cls.existing(
                id=name_or_id,
                connection=session._get_connection(),
                **params)
            return match.fetch(session, **params)
        except exceptions.NotFoundException:
            pass

        # NOTE(gtema) we do not do list, since previous call has done this
        # for us already

        if ignore_missing:
            return None
        raise exceptions.ResourceNotFound(
            "No %s found for %s" % (cls.__name__, name_or_id))


StackPreview = Stack
